const { localConfig } = require('./config');
const { projectsList } = require('./commands/projects');
const { functionsListRuntimes } = require('./commands/functions');
const { accountListMfaFactors } = require("./commands/account");
const { sdkForConsole } = require("./sdks");

const { databasesList } = require('./commands/databases');
const JSONbig = require("json-bigint")({ storeAsString: false });

const getIgnores = (runtime) => {
    const languge = runtime.split('-')[0];

    switch (languge) {
        case 'cpp':
            return ['build', 'CMakeFiles', 'CMakeCaches.txt'];
        case 'dart':
            return ['.packages', '.dart_tool'];
        case 'deno':
            return [];
        case 'dotnet':
            return ['bin', 'obj', '.nuget'];
        case 'java':
        case 'kotlin':
            return ['build'];
        case 'node':
        case 'bun':
            return ['node_modules', '.npm'];
        case 'php':
            return ['vendor'];
        case 'python':
            return ['__pypackages__'];
        case 'ruby':
            return ['vendor'];
        case 'rust':
            return ['target', 'debug', '*.rs.bk', '*.pdb'];
        case 'swift':
            return ['.build', '.swiftpm'];
    }

    return undefined;
};

const getEntrypoint = (runtime) => {
    const languge = runtime.split('-')[0];

    switch (languge) {
        case 'dart':
            return 'lib/main.dart';
        case 'deno':
            return 'src/main.ts';
        case 'node':
            return 'src/main.js';
        case 'bun':
            return 'src/main.ts';
        case 'php':
            return 'src/index.php';
        case 'python':
            return 'src/main.py';
        case 'ruby':
            return 'lib/main.rb';
        case 'rust':
            return 'main.rs';
        case 'swift':
            return 'Sources/index.swift';
        case 'cpp':
            return 'src/main.cc';
        case 'dotnet':
            return 'src/Index.cs';
        case 'java':
            return 'src/Main.java';
        case 'kotlin':
            return 'src/Main.kt';
    }

    return undefined;
};

const getInstallCommand = (runtime) => {
    const languge = runtime.split('-')[0];

    switch (languge) {
        case 'dart':
            return 'dart pub get';
        case 'deno':
            return "deno install";
        case 'node':
            return 'npm install';
        case 'bun':
            return 'bun install';
        case 'php':
            return 'composer install';
        case 'python':
            return 'pip install -r requirements.txt';
        case 'ruby':
            return 'bundle install';
        case 'rust':
            return 'cargo install';
        case 'dotnet':
            return 'dotnet restore';
        case 'swift':
        case 'java':
        case 'kotlin':
        case 'cpp':
            return '';
    }

    return undefined;
};

const questionsInitProject = [
    {
        type: "confirm",
        name: "override",
        message:
            `An Appwrite project ( ${localConfig.getProject()['projectName']} ) is already associated with the current directory. Would you like to override`,
        when() {
            return Object.keys(localConfig.getProject()).length !== 0;
        }
    },
    {
        type: "list",
        name: "start",
        when(answers) {
            if (answers.override == undefined) {
                return true
            }
            return answers.override;
        },
        message: "How would you like to start?",
        choices: [
            {
                name: "Create a new Appwrite project",
                value: "new",
            },
            {
                name: "Link this directory to an existing Appwrite project",
                value: "existing",
            },
        ],
    },
    {
        type: "input",
        name: "project",
        message: "What would you like to name your project?",
        default: "My Awesome Project",
        when(answers) {
            return answers.start == "new";
        },
    },
    {
        type: "input",
        name: "id",
        message: "What ID would you like to have for your project?",
        default: "unique()",
        when(answers) {
            return answers.start == "new";
        },
    },
    {
        type: "list",
        name: "project",
        message: "Choose your Appwrite project.",
        when(answers) {
            return answers.start == "existing";
        },
        choices: async () => {
            let response = await projectsList({
                parseOutput: false
            })
            let projects = response["projects"]
            let choices = projects.map((project, idx) => {
                return {
                    name: `${project.name} (${project['$id']})`,
                    value: {
                        name: project.name,
                        id: project['$id']
                    }
                }
            })

            if (choices.length == 0) {
                throw new Error("No projects found. Please create a new project.")
            }

            return choices;
        }
    }
];

const questionsInitFunction = [
    {
        type: "input",
        name: "name",
        message: "What would you like to name your function?",
        default: "My Awesome Function"
    },
    {
        type: "input",
        name: "id",
        message: "What ID would you like to have for your function?",
        default: "unique()"
    },
    {
        type: "list",
        name: "runtime",
        message: "What runtime would you like to use?",
        choices: async () => {
            let response = await functionsListRuntimes({
                parseOutput: false
            })
            let runtimes = response["runtimes"]
            let choices = runtimes.map((runtime, idx) => { 
                return {
                    name: `${runtime.name} (${runtime['$id']})`,
                    value: { 
                        id: runtime['$id'], 
                        entrypoint: getEntrypoint(runtime['$id']), 
                        ignore: getIgnores(runtime['$id']),
                        commands : getInstallCommand(runtime['$id'])
                    },
                }
            })
            return choices;
        }
    }
];

const questionsInitCollection = [
    {
        type: "checkbox",
        name: "databases",
        message: "From which database would you like to init collections?",
        choices: async () => {
            let response = await databasesList({
                parseOutput: false
            })
            let databases = response["databases"]

            if (databases.length <= 0) {
                throw new Error("No databases found. Please create one in project console.")
            }
            let choices = databases.map((database, idx) => {
                return {
                    name: `${database.name} (${database.$id})`,
                    value: database.$id
                }
            })
            return choices;
        }
    }
];

const questionsLogin = [
    {
        type: "input",
        name: "email",
        message: "Enter your email",
        validate(value) {
            if (!value) {
                return "Please enter your email";
            }
            return true;
        },
    },
    {
        type: "password",
        name: "password",
        message: "Enter your password",
        mask: "*",
        validate(value) {
            if (!value) {
                return "Please enter your password";
            }
            return true;
        }
    },
];

const questionsDeployFunctions = [
    {
        type: "checkbox",
        name: "functions",
        message: "Which functions would you like to deploy?",
        choices: () => {
            let functions = localConfig.getFunctions();
            if (functions.length === 0) {
                throw new Error("No functions found in the current directory.");
            }
            let choices = functions.map((func, idx) => {
                return {
                    name: `${func.name} (${func.$id})`,
                    value: func.$id
                }
            })
            return choices;
        }
    },
    {
        type: "input",
        name: "override",
        message: 'Are you sure you want to override this function\'s variables? This can lead to loss of secrets! Type "YES" to confirm.'
    },
]

const questionsDeployCollections = [
    {
        type: "checkbox",
        name: "collections",
        message: "Which collections would you like to deploy?",
        choices: () => {
            let collections = localConfig.getCollections();
            if (collections.length === 0) {
                throw new Error("No collections found in the current directory. Run `appwrite init collection` to fetch all your collections.");
            }
            return collections.map(collection => {
                return {
                    name: `${collection.name} (${collection['databaseId']} - ${collection['$id']})`,
                    value: `${collection['databaseId']}|${collection['$id']}`
                }
            });
        }
    },
    {
        type: "input",
        name: "override",
        message: 'Are you sure you want to override this collection? This can lead to loss of data! Type "YES" to confirm.'
    },
]

const questionsDeployBuckets = [
    {
        type: "checkbox",
        name: "buckets",
        message: "Which buckets would you like to deploy?",
        choices: () => {
            let buckets = localConfig.getBuckets();
            if (buckets.length === 0) {
                throw new Error("No buckets found in the current directory. Run `appwrite init bucket` to fetch all your buckets.");
            }
            return buckets.map(bucket => {
                return {
                    name: `${bucket.name} (${bucket['$id']})`,
                    value: bucket.$id
                }
            });
        }
    },
    {
        type: "input",
        name: "override",
        message: 'Are you sure you want to override this bucket? This can lead to loss of data! Type "YES" to confirm.'
    },
]

const questionsGetEntrypoint = [
    {
        type: "input",
        name: "entrypoint",
        message: "Enter the entrypoint",
        default: null,
        validate(value) {
            if (!value) {
                return "Please enter your entrypoint";
            }
            return true;
        }
    },
]

const questionsDeployTeams = [
    {
        type: "checkbox",
        name: "teams",
        message: "Which teams would you like to deploy?",
        choices: () => {
            let teams = localConfig.getTeams();
            if (teams.length === 0) {
                throw new Error("No teams found in the current directory. Run `appwrite init team` to fetch all your teams.");
            }
            return teams.map(team => {
                return {
                    name: `${team.name} (${team['$id']})`,
                    value: team.$id
                }
            });
        }
    },
    {
        type: "input",
        name: "override",
        message: 'Are you sure you want to override this team? This can lead to loss of data! Type "YES" to confirm.'
    },
];

const questionsListFactors = [
    {
        type: "list",
        name: "factor",
        message: "Your account is protected by multiple factors. Which factor would you like to use to authenticate?",
        choices: async () => {
            let client = await sdkForConsole(false);
            const factors = await accountListMfaFactors({
                sdk: client,
                parseOutput: false
            });
                
            const choices = [
                {
                    name: `TOTP (Time-based One-time Password)`,
                    value: 'totp'
                },
                {
                    name: `E-mail`,
                    value: 'email'
                },
                {
                    name: `Phone (SMS)`,
                    value: 'phone'
                },
                {
                    name: `Recovery code`,
                    value: 'recoveryCode'
                }
            ].filter((ch) => factors[ch.value] === true);

            return choices;
        }
    }
];

const questionsMfaChallenge = [
    {
        type: "input",
        name: "otp",
        message: "Enter OTP",
        validate(value) {
            if (!value) {
                return "Please enter OTP";
            }
            return true;
        },
    }
];

module.exports = {
    questionsInitProject,
    questionsLogin,
    questionsInitFunction,
    questionsInitCollection,
    questionsDeployFunctions,
    questionsDeployCollections,
    questionsDeployBuckets,
    questionsDeployTeams,
    questionsGetEntrypoint,
    questionsListFactors,
    questionsMfaChallenge
};
